use v5.40;
use Test2::V0;

use Minima;
use Minima::App;
use Minima::Controller;
use Minima::Project;
use Minima::Router;
use Minima::Setup;
use Minima::View::HTML;
use Minima::View::JSON;
use Minima::View::PlainText;
use Minima::View;

pass 'modules loaded';
done_testing;
